function BpodProtocol
% This protocol is the Go/NoGo protocol for Bpod
% Written by Malamati Bitzidou, 11/2016.
%
% SETUP
% What is needed:
% - A Pulse Pal with software installed on this computer and the
% alternative firmaware for Waveform Generator
% - A BNC cable between Bpod's BNC Output 1 and Pulse Pal's Trigger channel 1
% - A BNC cable between Pulse Pal's Output Cgannel 1 and a piezo
% - A port interface + valve connected to Bpod Port1

%% % Start Live Video
disp('Loading Webcam')
camlist = webcamlist;
if ~isempty(camlist)
    disp('Webcam Detected')
    vid = webcam;
    preview(vid)
else
disp('No Webcam Detected')
end 
%%
win_size = 50;
global BpodSystem

%% Initialize parameters and load stimuli

Fs = 12207; % Hz

RecordDur = 5;
MaxTrials = 600;
InterTrialInterval = 2.2;
BNCoutput = randi([0 1],MaxTrials,1);
for i=1:MaxTrials
    if BNCoutput(i,1)==0
        BNCoutput(i,1)=3;
    end
end

BpodSystem.Data.BNCoutput    = BNCoutput;
BpodSystem.Data.stimFile     = 'noises_feb17.mat';
load(BpodSystem.Data.stimFile)

%% BpodSystem extra parameters
BpodSystem.Data.LeftStim     = noises(4).stimulus; %4=Nogo, 3=zero , 8=short_zero
BpodSystem.Data.RightStim    = noises(5).stimulus; %5=Go, 7=a
BpodSystem.Data.Results.hit  = 0;
BpodSystem.Data.Results.miss = 0;
BpodSystem.Data.Results.cr   = 0;
BpodSystem.Data.Results.fp   = 0;
BpodSystem.Data.Results.dp   = 0;
BpodSystem.Data.Results.pcw  = 0;

% set pmtrs
StimulusDuration = length(BpodSystem.Data.LeftStim)/Fs;
WindowSize = 50;

%% Define Protocol Parameters 
S = BpodSystem.ProtocolSettings;
if isempty(fieldnames(S))  % If settings file was an empty struct, populate struct with default settings
    S.GUI.GoProb          = 0.5;
    S.GUI.MaxSame         = 3;
    S.GUI.TimeoutDuration = 5;
    S.GUI.pre_stim_time   = 0.2;
    S.GUI.DrinkTime       = 2;   
    S.GUI.GracePeriod     = 0.65;
    S.GUI.RespPeriod      = 1.5;
    S.GUI.lickvopen       = 0.034;
    S.GUI.RewardAmount    = 3.5; %ul
end

StimulusRemainder       = StimulusDuration - S.GUI.GracePeriod;
ResponsePeriodRemainder = S.GUI.RespPeriod - StimulusRemainder;

% Initialize parameter GUI plugin
BpodParameterGUI('init', S);

%% Initialize and program Pulse Pal
WaveGen                     = PulsePalWaveGen('COM11');
WaveGen.frequency           = Fs;
WaveGen.amplitude           = 5;
WaveGen.duration            = StimulusDuration;
WaveGen.playbackMode        = 'triggered';
WaveGen.LEDMode             = 'on';
WaveGen.activeOutputChannel = 1;
WaveGen.customWaveformSF    = Fs;

%% Define trials
BpodSystem.Data.TrialTypes = []; % The trial type of each trial completed will be added here

ae_image = zeros(2,MaxTrials,3);
end_ae = 0;
ae_divis = 1;
%% Main trial loop
CurrentTrial = 0;
while(CurrentTrial<MaxTrials)
    S = BpodParameterGUI('sync', S); % Sync params with BpodParameterGUI plugin
    R = GetValveTimes(S.GUI.RewardAmount, 1); ValveTime = R(1); % Update reward amounts
    CurrentTrial = CurrentTrial+1;
    
    % Determine if it's a Go or a NoGo trial
    if rand < S.GUI.GoProb        
        TrialTypes(CurrentTrial) = 1;        
        if CurrentTrial > S.GUI.MaxSame
            trialHistory = TrialTypes(CurrentTrial-S.GUI.MaxSame:CurrentTrial-1);
            if sum(trialHistory) == S.GUI.MaxSame
                TrialTypes(CurrentTrial) = 0;
                disp(['Max trials the same is ' num2str(S.GUI.MaxSame) ' and this has been reached'])
            end
        end
        
    else        
        TrialTypes(CurrentTrial) = 0;
        if CurrentTrial > S.GUI.MaxSame
            trialHistory = TrialTypes(CurrentTrial-S.GUI.MaxSame:CurrentTrial-1);
            if sum(trialHistory) == 0
                TrialTypes(CurrentTrial) = 1;
                disp(['Max trials the same is ' num2str(S.GUI.MaxSame) ' and this has been reached'])
            end
        end        
    end
    
    switch TrialTypes(CurrentTrial)
        case 1
            disp(['Trial ', num2str(CurrentTrial), ': It is a Go Trial'])
            LickActionState        = 'Reward';
            ValveCode              = 1;
            WaveGen.customWaveform = BpodSystem.Data.RightStim;
            WaveGen.waveform       = 'custom'; 
        case 0
            disp(['Trial ', num2str(CurrentTrial), ': It is a NoGo Trial'])
            LickActionState        = 'Punish1';
            ValveCode              = 1;
            WaveGen.customWaveform = BpodSystem.Data.LeftStim; 
            WaveGen.waveform       = 'custom'; 
    end    
    
    sma = NewStateMatrix(); % Create a blank matrix to define the trial's finite state machine
    
    sma = AddState(sma,'Name', 'InterTrialDelay',...
        'Timer', InterTrialInterval,...
        'StateChangeConditions', {'Tup', 'Start'},...
        'OutputActions', {});
    
    sma = AddState(sma,'Name', 'Start',...
        'Timer', S.GUI.GracePeriod,...
        'StateChangeConditions', {'Tup', 'WaitForResponse_Stim'},...
        'OutputActions', {'BNCState', 1}); % instead of 1, BNCoutput(CurrentTrial,1)
    
    sma = AddState(sma, 'Name', 'WaitForResponse_Stim', ...
        'Timer', StimulusRemainder, ...
        'StateChangeConditions', {'Tup', 'WaitForResponse', 'BNC1High', LickActionState}, ...
        'OutputActions', {'BNCState', 1});
    
    sma = AddState(sma, 'Name', 'WaitForResponse', ...
        'Timer', ResponsePeriodRemainder, ...
        'StateChangeConditions', {'BNC1High', LickActionState, 'Tup','exit'}, ...
        'OutputActions', {'BNCState', 0});
    
    sma = AddState(sma, 'Name', 'Reward', ...
        'Timer', ValveTime,...
        'StateChangeConditions', {'Tup', 'DrinkTime'},...
        'OutputActions', {'ValveState', ValveCode});
    
    sma = AddState(sma, 'Name', 'DrinkTime', ...
        'Timer', S.GUI.DrinkTime,...
        'StateChangeConditions', {'Tup', 'exit'},...
        'OutputActions', {'PWM1', 255});
    
    sma = AddState(sma, 'Name', 'Punish1', ...
        'Timer', S.GUI.TimeoutDuration,...
        'StateChangeConditions', {'BNC1High', 'Punish2','Tup','exit'},...
        'OutputActions', {});
    
    sma = AddState(sma, 'Name', 'Punish2', ...
        'Timer', S.GUI.TimeoutDuration,...
        'StateChangeConditions', {'BNC1High', 'Punish1','Tup','exit'},...
        'OutputActions', {});
    
    SendStateMatrix(sma);
    RawEvents = RunStateMatrix;
       if ~isempty(fieldnames(RawEvents)) % If trial data was returned
            BpodSystem.Data = AddTrialEvents(BpodSystem.Data,RawEvents); % Computes trial events from raw data
            BpodSystem.Data.TrialSettings(CurrentTrial) = S; % Adds the settings used for the current trial to the Data struct (to be saved after the trial ends)
            BpodSystem.Data.TrialTypes(CurrentTrial) = TrialTypes(CurrentTrial); % Adds the trial type of the current trial to data
            
%%           Compute performance metrics
%            log structure:   hit = 1
%                             miss = 2
%                             cr = 3
%                             fp = 4

            maxState = max(BpodSystem.Data.RawData.OriginalStateData{1,CurrentTrial});
            finalState = BpodSystem.Data.RawData.OriginalStateNamesByNumber{1,CurrentTrial}{1,maxState};
            
            if TrialTypes(CurrentTrial) == 1
            %%%%%%%%%%%%%%%%%%%% GO
                if strcmp(finalState,'DrinkTime') %hit
                    BpodSystem.Data.Results.hit = BpodSystem.Data.Results.hit+1;
                    temp_result = 1;
                else %miss
                   BpodSystem.Data.Results.miss = BpodSystem.Data.Results.miss+1;
                    temp_result = 2;
                end   
            else
            %%%%%%%%%%%%%%%%%%%% NOGO
                if strcmp(finalState,'Punish1') || strcmp(finalState,'Punish2') %false acceptance 
                    BpodSystem.Data.Results.fp = BpodSystem.Data.Results.fp+1;
                    temp_result = 4;
                else %correct rejection
                    BpodSystem.Data.Results.cr = BpodSystem.Data.Results.cr+1;
                    temp_result = 3;    
                end
            end
            
            BpodSystem.Data.Results.log(CurrentTrial,:) = zeros(1,4);
            BpodSystem.Data.Results.log(CurrentTrial,temp_result) = 1;
            BpodSystem.Data.Results.hr = BpodSystem.Data.Results.hit/sum([BpodSystem.Data.Results.hit BpodSystem.Data.Results.miss]);
            BpodSystem.Data.Results.far = BpodSystem.Data.Results.fp/sum([BpodSystem.Data.Results.cr BpodSystem.Data.Results.fp]);
            BpodSystem.Data.Results.pc = sum([BpodSystem.Data.Results.hit BpodSystem.Data.Results.cr])/CurrentTrial;
% 
            if CurrentTrial>WindowSize-1
                win = CurrentTrial-(win_size-1):CurrentTrial;
            else
                win = 1:CurrentTrial;
            end
                        
            nGow = sum(BpodSystem.Data.Results.log(win,1))+sum(BpodSystem.Data.Results.log(win,2));
            nNoGow = sum(BpodSystem.Data.Results.log(win,3))+sum(BpodSystem.Data.Results.log(win,4));
            pHitw = sum(BpodSystem.Data.Results.log(win,1))/nGow;
            BpodSystem.Data.Results.hrw = pHitw;
            pCrw = sum(BpodSystem.Data.Results.log(win,3))/nNoGow;            
            BpodSystem.Data.Results.farw = 1-pCrw;
            BpodSystem.Data.Results.pcww = (pHitw+pCrw)/2;
            BpodSystem.Data.Results.pcw = (sum(BpodSystem.Data.Results.log(win,1))+sum(BpodSystem.Data.Results.log(win,3)))/length(win);
            disp(BpodSystem.Data.Results.pcw)
            BpodSystem.Data.Results.dpw = norminv(BpodSystem.Data.Results.hr,0,1)-norminv(BpodSystem.Data.Results.far,0,1);
            
            SaveBpodSessionData; % Saves the field BpodSystem.Data to the current data file
% PLOTING
L = length(WaveGen.customWaveform)/Fs+S.GUI.pre_stim_time+S.GUI.RespPeriod; %in seconds
plot_sequence
        end
        HandlePauseCondition;
    if BpodSystem.Status.BeingUsed == 0
%         figure;
%         plotSessionData
        return 
    end
    clear LickActionState    
end
end_ae = 1;





